#include <lemon/lp.h>
#include "test_tools.h"

using namespace lemon;

void solveAndCheck(Mip& lp, LpSolverBase::SolutionStatus stat, 
		   double exp_opt) {
  using std::string;
  lp.solve();
  //int decimal,sign;
  std::ostringstream buf;
  buf << "Primalstatus should be: " << int(stat)<<" and it is "<<int(lp.primalStatus());

  //  itoa(stat,buf1, 10);
  check(lp.primalStatus()==stat, buf.str());

  if (stat ==  LpSolverBase::OPTIMAL) {
    std::ostringstream buf;
    buf << "Wrong optimal value: the right optimum is " << exp_opt; 
    check(std::abs(lp.primalValue()-exp_opt) < 1e-3, buf.str());
    //+ecvt(exp_opt,2)
  }
}

void aTest(Mip& mip)
{
 //The following example is very simple

  typedef Mip::Row Row;
  typedef Mip::Col Col;


  Col x1 = mip.addCol();
  Col x2 = mip.addCol();



  //Constraints
  mip.addRow(2*x1+x2 <=2);  
  mip.addRow(x1-2*x2 <=0);  

  //Nonnegativity of the variable x1
  mip.colLowerBound(x1, 0);



  //Objective function
  mip.setObj(x1);

  mip.max();


  //Maximization of x1
  //over the triangle with vertices
  double expected_opt=4.0/5.0;
  solveAndCheck(mip, Mip::OPTIMAL, expected_opt);

  //Restrict x2 to integer
  mip.colType(x2,Mip::INTEGER);  
  expected_opt=1.0/2.0;
  solveAndCheck(mip, Mip::OPTIMAL, expected_opt);


  //Restrict both to integer
  mip.colType(x1,Mip::INTEGER);  
  expected_opt=0;
  solveAndCheck(mip, Mip::OPTIMAL, expected_opt);

 

}


int main() 
{

#ifdef HAVE_GLPK
  MipGlpk mip1;
  aTest(mip1);
#endif

  return 0;

}
